from functools import wraps
from quart import g

from foxnouns.exceptions import ForbiddenError, ErrorCode


def require_auth(*, scope: str | None = None):
    def decorator(func):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            if not ("user" in g) or not ("token" in g):
                raise ForbiddenError("Not authenticated", type=ErrorCode.Forbidden)

            if scope and not g.token.has_scope(scope):
                raise ForbiddenError(
                    f"Missing scope '{scope}'", type=ErrorCode.MissingScope
                )

            return await func(*args, **kwargs)

        return wrapper

    return decorator
